/*
 * mysql_costcalc_load.js
 *
 * [Oracle Database]
 * shell> sqlplus "/ AS SYSDBA"
 * sql> CREATE USER scott IDENTIFIED BY tiger;
 * sql> GRANT connect, resource TO scott;
 *
 * [MySQL]
 * shell> mysql -u root [-p]
 * sql> CREATE DATABASE scott;
 * sql> GRANT ALL PRIVILEGES ON scott.* TO scott@'%' IDENTIFIED BY 'tiger';
 *
 * [PostgreSQL]
 * shell> psql -U postgres
 * sql> CREATE DATABASE scott;
 * sql> CREATE USER scott PASSWORD 'tiger';
 *
 * <postgresql.conf>
 * listen_addresses = '*'
 * port = 5432
 *
 * <pg_hba.conf>
 * host all all 0.0.0.0/0 md5
 */

// JdbcRunner settings -----------------------------------------------

// Oracle Database
// var jdbcUrl = "jdbc:oracle:thin://@localhost:1521/ORCL";

// MySQL
var jdbcUrl = "jdbc:mysql://localhost:3306/scott?rewriteBatchedStatements=true";

// PostgreSQL
// var jdbcUrl = "jdbc:postgresql://localhost:5432/scott";

var jdbcUser = "scott";
var jdbcPass = "tiger";
var isLoad = true;
var nAgents = 1;
var isAutoCommit = false;
var isDebug = false;
var isTrace = false;
var logDir = "logs";

// Application settings ----------------------------------------------

var BATCH_SIZE = 100;
var COMMIT_SIZE = 1000;
var PRINT_SIZE = 10000;

var ALPHA_NUMERIC_2;
var ALPHA_NUMERIC = [
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
    'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
    'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'];

// JdbcRunner functions ----------------------------------------------

function init() {
    if (getId() == 0) {
        dropTable();
        
        if (getDatabaseProductName() == "Oracle") {
            createTableOracle();
        } else if (getDatabaseProductName() == "MySQL") {
            createTableMySQL();
        } else if (getDatabaseProductName() == "PostgreSQL") {
            createTablePostgreSQL();
        } else {
            error(getDatabaseProductName() + " is not supported yet.");
        }
        
        commit();
    }
}

function run() {
    if (getId() == 0) {
        loadCustomer();
        loadItem();
        loadOrders();
        loadOrderLine();
    }
    
    setBreak();
}

function fin() {
    if (getId() == 0) {
        if (getDatabaseProductName() == "Oracle") {
            gatherStatsOracle();
        } else if (getDatabaseProductName() == "MySQL") {
            // Do nothing.
        } else if (getDatabaseProductName() == "PostgreSQL") {
            gatherStatsPostgreSQL();
        } else {
            error(getDatabaseProductName() + " is not supported yet.");
        }
        
        commit();
        info("Completed.");
    }
}

// Application functions ---------------------------------------------

function dropTable() {
    info("Dropping tables ...");
    
    dropTableByName("order_line");
    dropTableByName("orders");
    dropTableByName("item");
    dropTableByName("customer");
}

function dropTableByName(tableName) {
    try {
        execute("DROP TABLE " + tableName);
    } catch (e) {
        warn(e);
        rollback(); // PostgreSQL requires a rollback.
    }
}

function createTableOracle() {
    info("Creating tables ...");
    
    execute("CREATE TABLE customer (c_id NUMBER, c_name VARCHAR2(100), c_birth TIMESTAMP)");
    execute("CREATE TABLE item (i_id NUMBER, i_name VARCHAR2(100), i_price NUMBER)");
    execute("CREATE TABLE orders (o_id NUMBER, c_id NUMBER, o_date TIMESTAMP)");
    execute("CREATE TABLE order_line (o_id NUMBER, ol_id NUMBER, i_id NUMBER, ol_quantity NUMBER)");
    
    execute("ALTER TABLE customer ADD CONSTRAINT customer_pk PRIMARY KEY (c_id)");
    execute("ALTER TABLE item ADD CONSTRAINT item_pk PRIMARY KEY (i_id)");
    execute("ALTER TABLE orders ADD CONSTRAINT orders_pk PRIMARY KEY (o_id)");
    execute("ALTER TABLE order_line ADD CONSTRAINT order_line_pk PRIMARY KEY (o_id, ol_id)");
}

function createTableMySQL() {
    info("Creating tables ...");
    
    execute("CREATE TABLE customer (c_id INT PRIMARY KEY, c_name VARCHAR(100), c_birth DATETIME)");
    execute("CREATE TABLE item (i_id INT PRIMARY KEY, i_name VARCHAR(100), i_price INT)");
    execute("CREATE TABLE orders (o_id INT PRIMARY KEY, c_id INT, o_date DATETIME)");
    execute("CREATE TABLE order_line (o_id INT, ol_id INT, i_id INT, ol_quantity INT, PRIMARY KEY (o_id, ol_id))");
}

function createTablePostgreSQL() {
    info("Creating tables ...");
    
    execute("CREATE TABLE customer (c_id INTEGER, c_name VARCHAR(100), c_birth TIMESTAMP)");
    execute("CREATE TABLE item (i_id INTEGER, i_name VARCHAR(100), i_price INTEGER)");
    execute("CREATE TABLE orders (o_id INTEGER, c_id INTEGER, o_date TIMESTAMP)");
    execute("CREATE TABLE order_line (o_id INTEGER, ol_id INTEGER, i_id INTEGER, ol_quantity INTEGER)");
    
    execute("ALTER TABLE customer ADD CONSTRAINT customer_pk PRIMARY KEY (c_id)");
    execute("ALTER TABLE item ADD CONSTRAINT item_pk PRIMARY KEY (i_id)");
    execute("ALTER TABLE orders ADD CONSTRAINT orders_pk PRIMARY KEY (o_id)");
    execute("ALTER TABLE order_line ADD CONSTRAINT order_line_pk PRIMARY KEY (o_id, ol_id)");
}

function gatherStatsOracle() {
    info("Analyzing tables ...");
    
    execute("BEGIN DBMS_STATS.GATHER_SCHEMA_STATS(ownname => NULL); END;");
}

function gatherStatsPostgreSQL() {
    info("Vacuuming and analyzing tables ...");
    
    takeConnection().setAutoCommit(true);
    execute("VACUUM ANALYZE customer");
    execute("VACUUM ANALYZE item");
    execute("VACUUM ANALYZE orders");
    execute("VACUUM ANALYZE order_line");
    takeConnection().setAutoCommit(false);
}

function loadCustomer() {
    info("Loading customer ...");
    
    var c_id = new Array(BATCH_SIZE);
    var c_name = new Array(BATCH_SIZE);
    var c_birth =  new Array(BATCH_SIZE);
    
    for (var customerId = 1; customerId <= 10000; customerId++) {
        var index = (customerId - 1) % BATCH_SIZE;
        
        c_id[index] = customerId;
        c_name[index] = randomString(15, 30);
        c_birth[index] = randomDate(100);
        
        if (customerId % BATCH_SIZE == 0) {
            executeBatch("INSERT INTO customer (c_id, c_name, c_birth) VALUES ($int, $string, $timestamp)",
                c_id, c_name, c_birth);
            
            if (customerId % COMMIT_SIZE == 0) {
                commit();
                
                if (customerId % PRINT_SIZE == 0) {
                    info("customer : " + customerId + " / 10000");
                }
            }
        }
    }
}

function loadItem() {
    info("Loading item ...");
    
    var i_id = new Array(BATCH_SIZE);
    var i_name = new Array(BATCH_SIZE);
    var i_price = new Array(BATCH_SIZE);
    
    for (var itemId = 1; itemId <= 100000; itemId++) {
        var index = (itemId - 1) % BATCH_SIZE;
        
        i_id[index] = itemId;
        i_name[index] = randomString(10, 20);
        i_price[index] = random(1, 100000);
        
        if (itemId % BATCH_SIZE == 0) {
            executeBatch("INSERT INTO item (i_id, i_name, i_price) VALUES ($int, $string, $int)",
                i_id, i_name, i_price);
            
            if (itemId % COMMIT_SIZE == 0) {
                commit();
                
                if (itemId % PRINT_SIZE == 0) {
                    info("item : " + itemId + " / 100000");
                }
            }
        }
    }
}

function loadOrders() {
    info("Loading orders ...");
    
    var o_id = new Array(BATCH_SIZE);
    var c_id = new Array(BATCH_SIZE);
    var o_date = new Array(BATCH_SIZE);
    
    for (var orderId = 1; orderId <= 50000; orderId++) {
        var index = (orderId - 1) % BATCH_SIZE;
        
        o_id[index] = orderId;
        c_id[index] = random(1, 10000);
        o_date[index] = randomDate(1);
        
        if (orderId % BATCH_SIZE == 0) {
            executeBatch("INSERT INTO orders (o_id, c_id, o_date) VALUES ($int, $int, $timestamp)",
                o_id, c_id, o_date);
            
            if (orderId % COMMIT_SIZE == 0) {
                commit();
                
                if (orderId % PRINT_SIZE == 0) {
                    info("order : " + orderId + " / 50000");
                }
            }
        }
    }
}

function loadOrderLine() {
    info("Loading order_line ...");
    
    var o_id = new Array();
    var ol_id = new Array();
    var i_id = new Array();
    var ol_quantity = new Array();
    
    for (var orderId = 1; orderId <= 50000; orderId++) {
        var orderCount = random(1, 5);
        
        for (var orderLineId = 1; orderLineId <= orderCount; orderLineId++) {
            o_id.push(orderId);
            ol_id.push(orderLineId);
            i_id.push(random(1, 100000));
            ol_quantity.push(random(1, 10));
        }
        
        if (orderId % BATCH_SIZE == 0) {
            executeBatch("INSERT INTO order_line (o_id, ol_id, i_id, ol_quantity) VALUES ($int, $int, $int, $int)",
                o_id, ol_id, i_id, ol_quantity);
            
            o_id.length = 0;
            ol_id.length = 0;
            i_id.length = 0;
            ol_quantity.length = 0;
            
            if (orderId % COMMIT_SIZE == 0) {
                commit();
                
                if (orderId % PRINT_SIZE == 0) {
                    info("order_line : " + orderId + " / 50000");
                }
            }
        }
    }
}

function randomString(minLength, maxLength) {
    if (!ALPHA_NUMERIC_2) {
        ALPHA_NUMERIC_2 = new Array(3844);
        
        for (var i = 0; i < 62; i++) {
            for (var j = 0; j < 62; j++) {
                ALPHA_NUMERIC_2[i * 62 + j] = ALPHA_NUMERIC[i] + ALPHA_NUMERIC[j];
            }
        }
    }
    
    var length = random(minLength, maxLength);
    var index = 0;
    var rand = 0;
    var rest = 0;
    var string = "";
    
    while (index < length) {
        if (rest == 0) {
            rand = random(0, 218340105584895);
            rest = 8;
        }
        
        if (index + 1 < length) {
            var code = rand % 3844;
            string += ALPHA_NUMERIC_2[code];
            rand = (rand - code) / 3844;
            rest -= 2;
            index += 2;
        } else {
            var code = rand % 62;
            string += ALPHA_NUMERIC[code];
            break;
        }
    }
    
    return string;
}

function randomDate(range) {
    var date = new Date(2012, 0, 1, 0, 0, 0);
    var diff = random(1, range * 365 * 24 * 60 * 60 * 1000);
    
    date.setTime(date.getTime() - diff);
    
    return date;
}
